package com.sun.xml.internal.ws.server.provider;

import java.util.Iterator;
import java.util.Map;
import javax.xml.soap.MimeHeaders;
import java.util.ArrayList;
import java.util.List;
import javax.xml.soap.MimeHeader;
import java.util.HashMap;
import com.sun.xml.internal.ws.api.WSBinding;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLPort;
import com.sun.istack.internal.Nullable;
import javax.xml.soap.SOAPException;
import com.sun.xml.internal.ws.fault.SOAPFaultBuilder;
import com.sun.xml.internal.ws.api.message.Messages;
import com.sun.xml.internal.ws.api.message.Packet;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.resources.ServerMessages;
import com.sun.xml.internal.ws.api.message.Message;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.Source;
import javax.xml.ws.Service;
import com.sun.xml.internal.ws.api.SOAPVersion;

abstract class SOAPProviderArgumentBuilder<T> extends ProviderArgumentsBuilder<T>
{
    protected final SOAPVersion soapVersion;
    
    private SOAPProviderArgumentBuilder(SOAPVersion soapVersion) {
        this.soapVersion = soapVersion;
    }
    
    static ProviderArgumentsBuilder create(ProviderEndpointModel model, SOAPVersion soapVersion) {
        if (model.mode == Service.Mode.PAYLOAD) {
            return new PayloadSource(soapVersion);
        }
        if (model.datatype == Source.class) {
            return new MessageSource(soapVersion);
        }
        if (model.datatype == SOAPMessage.class) {
            return new SOAPMessageParameter(soapVersion);
        }
        if (model.datatype == Message.class) {
            return new MessageProviderArgumentBuilder(soapVersion);
        }
        throw new WebServiceException(ServerMessages.PROVIDER_INVALID_PARAMETER_TYPE(model.implClass, model.datatype));
    }
    
    SOAPProviderArgumentBuilder(SOAPVersion x0, SOAPProviderArgumentBuilder.1 x1) {
        this(x0);
    }
    
    private static final class PayloadSource extends SOAPProviderArgumentBuilder<Source>
    {
        PayloadSource(SOAPVersion soapVersion) {
            super(soapVersion, null);
        }
        
        @Override
        public Source getParameter(Packet packet) {
            return packet.getMessage().readPayloadAsSource();
        }
        
        @Override
        protected Message getResponseMessage(Source source) {
            return Messages.createUsingPayload(source, this.soapVersion);
        }
        
        @Override
        protected Message getResponseMessage(Exception e) {
            return SOAPFaultBuilder.createSOAPFaultMessage(this.soapVersion, null, e);
        }
    }
    
    private static final class MessageSource extends SOAPProviderArgumentBuilder<Source>
    {
        MessageSource(SOAPVersion soapVersion) {
            super(soapVersion, null);
        }
        
        @Override
        public Source getParameter(Packet packet) {
            return packet.getMessage().readEnvelopeAsSource();
        }
        
        @Override
        protected Message getResponseMessage(Source source) {
            return Messages.create(source, this.soapVersion);
        }
        
        @Override
        protected Message getResponseMessage(Exception e) {
            return SOAPFaultBuilder.createSOAPFaultMessage(this.soapVersion, null, e);
        }
    }
    
    private static final class SOAPMessageParameter extends SOAPProviderArgumentBuilder<SOAPMessage>
    {
        SOAPMessageParameter(SOAPVersion soapVersion) {
            super(soapVersion, null);
        }
        
        @Override
        public SOAPMessage getParameter(Packet packet) {
            try {
                return packet.getMessage().readAsSOAPMessage(packet, true);
            }
            catch (SOAPException se) {
                throw new WebServiceException(se);
            }
        }
        
        @Override
        protected Message getResponseMessage(SOAPMessage soapMsg) {
            return Messages.create(soapMsg);
        }
        
        @Override
        protected Message getResponseMessage(Exception e) {
            return SOAPFaultBuilder.createSOAPFaultMessage(this.soapVersion, null, e);
        }
        
        @Override
        protected Packet getResponse(Packet request, @Nullable SOAPMessage returnValue, WSDLPort port, WSBinding binding) {
            Packet response = super.getResponse(request, returnValue, port, binding);
            if (returnValue != null && response.supports("com.sun.xml.internal.ws.api.message.packet.outbound.transport.headers")) {
                MimeHeaders hdrs = returnValue.getMimeHeaders();
                Map<String, List<String>> headers = new HashMap();
                Iterator<Object> i = hdrs.getAllHeaders();
                while (i.hasNext()) {
                    MimeHeader header = (MimeHeader)i.next();
                    if (header.getName().equalsIgnoreCase("SOAPAction")) {
                        continue;
                    }
                    List<String> list = (List)headers.get(header.getName());
                    if (list == null) {
                        list = new ArrayList();
                        headers.put(header.getName(), list);
                    }
                    list.add(header.getValue());
                }
                response.put("com.sun.xml.internal.ws.api.message.packet.outbound.transport.headers", headers);
                return response;
            }
            return response;
        }
    }
}
